<?php

use \StructuredDynamics\structwsf\php\api\ws\auth\lister\AuthListerQuery;
use \StructuredDynamics\structwsf\php\api\framework\ServerIDQuery;
/**
 * Form builder; Form for editing a construct endpoint.
 * Used by CTools export ui
 *
 * @ingroup forms
 */
function structconf_dataset_form(&$form, &$form_state) {

  // Get URI and Info from query string when present
  if($form_state['op'] == 'add' && isset($_GET['uri'])) {
    $uri = check_plain(urldecode($_GET['uri']));
    
    $uri = structconf_process_uri_or_path($uri);

    // This is a cached call specific to the user
    $info = structconf_api_fetch_dataset_info($uri);    
    $form['info']['label']['#default_value'] = $info['prefLabel'];
  }

  $preset = $form_state['item'];

  $form['#attached']['css'] = array(
    drupal_get_path('module', 'structconf') . '/structconf.css',
  );

  /************* Datasets ***********
    * structconf_datasets
    ** scdid
    ** label
    ** machine_name
    ** origin
    ** active
    ** sceid (endpoint)
    ** scbu_mn (base_uri_machine_name)
    ** uri_path (optional)
    ** remote_label
    ** remote_description
    */

  $form['active'] = array(
    '#type' => 'checkbox',
    '#title' => t('Dataset is searchable'),
    '#description' => t('The dataset will be exposed to the structSearchAPI module by checking this option. If the dataset is not marked as <em>searchable</em>, it will be be of type <em>normal</em> which mean that it will be available to other modules.'),
    '#default_value' => $preset->active,
  );

  $form['config'] = array(
    '#type' => 'fieldset',
    '#title' => t('Dataset URI Configuration'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,    
  );

  $origin_default = empty($preset->origin) ? 'shared' : $preset->origin;
  $form['origin'] = array('#type' => 'hidden', '#value' => $origin_default);  
  
  $sceid_default = empty($preset->sceid) ? current(structconf_get_default_endpoint())->sceid : $preset->sceid;
  $form['config']['sceid'] = array('#type' => 'hidden', '#value' => $sceid_default);  

  $form['config']['warning']['#markup'] = t('<br /><strong>WARNING:</strong> Be careful when updating this information. Make sure that the dataset URI you are configuring here is existing in the structWSF instance. Otherwise, it may lead to unexisting datasets errors elsewhere in the portal!<br /><br />');
  
  if(isset($uri) && $uri) {
    $form['config']['full_uri'] = array('#type' => 'hidden', '#value' => $uri);      
  }
  else {
    $form['config']['scbu_mn'] = array(
      '#type' => 'select',
      '#title' => t('Dataset Base URI'),
      '#description' => t('This Base URI will be used to access the dataset.'),
      '#default_value' => $preset->scbu_mn,
      '#options' => structconf_get_base_uris_form_options(),
    );

    $form['config']['seperator'] = array(
      '#markup' => '<div class="separator"> + </div>',
    );

    $form['config']['uri_path'] = array(
      '#type' => 'textfield',
      '#title' => t('URI Path'),
      '#description' => t('The URI Path to be appended to the base URI.'),
      '#default_value' => $preset->uri_path,
    );
  }  
  
  /*
  //$authLister = new $authLister();

  $header = array();  
  $header['role'] = array('data' => t('Role'));
  $header['create'] = array('data' => t('Create'));
  $header['read'] = array('data' => t('Read'));
  $header['update'] = array('data' => t('Update'));
  $header['delete'] = array('data' => t('Delete'));
  $header['operations'] = array('data' => t('Operations'));
 
  $operations = array(); 
  $operations[] = l(t('edit'), 'admin/config/construct/networks_and_dataset/datasets/permissions/'.$role.'/edit');
  $operations[] = l(t('delete'), 'admin/config/construct/networks_and_dataset/datasets/permissions/'.$role.'/delete');
  
  $form['permissions']['table'] = array(
    '#type' => 'tableselect',
    '#weight' => '3',
    '#header' => $header,
    '#options' => array(),
    '#empty' => t('No content available.'),
  );
  
  $form['permissions'] = array(
    '#type' => 'fieldset',
    '#title' => t('Permissions'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );   
       
  */     
  
  $form['remote_updates'] = array(
    '#type' => 'fieldset',
    '#title' => t('Remote Updates'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['remote_updates']['warning']['#markup'] = t('<strong>WARNING:</strong> Updates to these fields will be reflected on any sites using the same API Endpoint and dataset!');

  $form['remote_updates']['remote_label'] = array(
    '#type' => 'textfield',
    '#title' => t('Label'),
    '#description' => t('The Label of the dataset. To be saved to the CCR endpoint.'),
    '#default_value' => (isset($info) ? $info['prefLabel'] : ''),
  );

  $form['remote_updates']['remote_description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('The Description of the dataset. To be saved to the CCR endpoint.'),
    '#default_value' => (isset($info) ? $info['description'] : ''),
  ); 
  
  return $form;
}

function structconf_dataset_form_validate(&$form, &$form_state) {
  if(isset($form_state['values']['full_uri'])){
    $no_match = TRUE;
    $base_uris = structconf_base_uri_load_multiple();
    $full_uri = $form_state['values']['full_uri'];
    $longest_base_uri_len = 0;
    foreach($base_uris as $base_uri) {
      if(strpos($full_uri, $base_uri->uri) === 0 && strlen($base_uri->uri) > $longest_base_uri_len) {
        // Use existing base URI
        $longest_base_uri_len = strlen($base_uri->uri);
        $no_match = FALSE;
        $form_state['values']['scbu_mn'] = $base_uri->machine_name;
        $form_state['values']['uri_path'] = substr($full_uri, strlen($base_uri->uri));
        drupal_set_message('Used existing Base URI');
      }
    }
    if($no_match) {
      // Create a new Base URI programatically
      $baseUriObject = structconf_base_uri_object($full_uri);      
      $new_base_uri_record = structconf_base_uri_save($baseUriObject);
      $form_state['values']['scbu_mn'] = $new_base_uri_record->machine_name;
      drupal_set_message('Created new Base URI');
    }
  }

  $base_uri_record = current(structconf_base_uri_load_multiple(array($form_state['values']['scbu_mn'])));

  $form_state['values']['uri_path'] = '';
  $form_state['values']['uri_path'] = structconf_process_uri_or_path($form_state['values']['uri_path']);
  $uri = $base_uri_record->uri .'/'. $form_state['values']['uri_path'];
  if (structconf_uri_is_invalid($uri)) {
    form_set_error('uri_path', t('This path results in an improperly formatted URL.'));
  }
}

/**
 * Form builder; Form for editing a construct endpoint.
 * Used by CTools export ui
 *
 * @ingroup forms
 */
function structconf_endpoint_form(&$form, &$form_state) {
  $preset = $form_state['item'];

  $form['uri'] = array(
    '#type' => 'textfield',
    '#title' => t('URI'),
    '#description' => t('The full URI of the endpoint'),
    '#default_value' => $preset->uri,
    '#required' => true,
  );

  // @TODO Use testfield when color picker module isn't present
  $form['color'] = array(
    '#type' => 'jquery_colorpicker',
    '#title' => t('Color'),
    '#default_value' => $preset->color,
    '#description' => t('Color coding for managing this endpoint.'),
    '#required' => true,
  );

  $form['is_default'] = array(
    '#type' => 'checkbox',
    '#title' => t('Default'),
    '#description' => t('If set, this endpoint will be used as the default.'),
    '#default_value' => $preset->is_default,
  );

  $form['active'] = array(
    '#type' => 'checkbox',
    '#title' => t('Active'),
    '#description' => t('If set, this endpoint will be used simultaneously with all others.'),
    '#default_value' => $preset->active,
  );

  return $form;
}

function structconf_endpoint_form_validate(&$form, &$form_state) {
  // Get the SID of the endpoint and save it into structConf
  // This also act as a way to make sure that the endpoint is up, running and operational  
  $serverIDQuery = new ServerIDQuery($form_state['values']['uri']);
  
  $serverIDQuery->send(new DrupalQuerierExtension());
  
  if($serverIDQuery->isSuccessful())
  {
    $sid = $serverIDQuery->getResultset();
    if(empty($sid))
    {
      form_set_error('sid', t('The server ID (SID) returned by the server is empty which means that the endpoint is not properly configured. Make sure the endpoint is properly configured and try again.'));
      return;
    }
    else
    {
      $form_state['item']->{'sid'} = $sid;
    }    
  }
  else
  {
    form_set_error('uri', t('It is not possible to get the server ID (SID) for that endpoint. Make sure the endpoint is properly functioning and try again to register it.'));
    return;
  }

  $form_state['values']['uri'] = structconf_process_uri_or_path($form_state['values']['uri']);
  
  // Be a bit less strict with this one
  if (!valid_url($form_state['values']['uri'])) {
    form_set_error('uri', t('Improperly formatted URI.'));
  }
}

function structconf_endpoint_form_submit(&$form, &$form_state) {
  if($form_state['input']['is_default']) {
    // Default must be active
    $form_state['input']['active'] = "1";
    foreach (structconf_get_endpoints() as $endpoint) {
      if ($endpoint->is_default === "1" && $endpoint->sceid !== STRUCTCONF_ENDPOINT_DEFAULT_SCEID) {
        $endpoint->label = str_replace('Active Default:', '', $endpoint->label);
        $endpoint->is_default = "0";
        structconf_endpoint_save($endpoint);
      }
    }
  }
  // Clear the Ctools export API cache.
  ctools_include('export');
  ctools_export_load_object_reset('structconf_endpoints');
  ctools_export_load_object_reset('structconf_datasets');
  
  // Clear all cache
  drupal_flush_all_caches();
}


/**
 * Form builder; Form for editing a construct base_uris.
 * Used by CTools export ui
 *
 * @ingroup forms
 */
function structconf_base_uri_form(&$form, &$form_state) {
  $preset = $form_state['item'];

  $form['uri'] = array(
    '#type' => 'textfield',
    '#title' => t('URI'),
    '#description' => t('The base URI some datasets'),
    '#default_value' => $preset->uri,
    '#required' => true,
  );

  return $form;
}

function structconf_base_uri_form_validate(&$form, &$form_state) {
   $form_state['values']['uri'] = structconf_process_uri_or_path($form_state['values']['uri']);
   if (structconf_uri_is_invalid($form_state['values']['uri'])) {
     form_set_error('uri', t('Improperly formatted URI.'));
   }
}
