<?php
  
include_once DRUPAL_ROOT . '/' . drupal_get_path('module', 'construct') . '/framework/ClassHierarchy.php';

use \StructuredDynamics\structwsf\php\api\ws\search\SearchQuery;
use \StructuredDynamics\structwsf\framework\Namespaces;
use \StructuredDynamics\structwsf\framework\Resultset;
use \StructuredDynamics\structwsf\framework\Subject;
use \StructuredDynamics\structwsf\ws\framework\ClassHierarchy;


/**
* 
*  Code related to the UI part of this module used to do the mapping between fields and OWL properties
* 
*  Most of this code come from the RDFUI module part of the RDFX project
* 
*/

/**
 * Implements hook_help().
 */
function structfieldstorage_help($path, $arg) {
  switch ($path) {
    case 'admin/structure/types/manage/%/rdf':
      return '<p>' . t('Manage the way this bundle and its fields are represented in RDF. The mappings defined here will be used to save the bundle\'s content into a structWSF instance', array()) . '</p>';
  }
}

/**
 * Implements hook_theme().
 */
function structfieldstorage_theme($path, $arg) {
  return array(
  'structfieldstorage_term_autocomplete' => array(
      'render element' => 'term',
    ),
  );
}

/**
 * Implements hook_permission().
 */
function structfieldstorage_permission() {
  return array(
    'administer RDF field mappings for conStruct' => array(
      'title' => t('Change the RDF mappings for types and fields for conStruct'),
    ),
  );
}

/**
 * Menu callback; listing of field RDF mappings for a content type.
 *
 * Allows the content type to be mapped to a RDF class and
 * fields to be mapped to RDF properties.
 */
function structfieldstorage_admin_rdf_overview_form($form, &$form_state, $entity_type, $bundle, $instance) {
  $bundle = field_extract_bundle($entity_type, $bundle);
  $fields = field_info_instances($entity_type, $bundle);
  $mapping = rdf_mapping_load($entity_type, $bundle);
  
  // Don't make assumptions about entities and only get the type if dealing with a node entity
  $type = ($entity_type == 'node') ? node_type_get_type($bundle) : '';
  $form['rdf_mappings'] = array(
    '#type' => 'vertical_tabs',
    '#attached' => array(
      'js' => array(
        drupal_get_path('module', 'structfieldstorage') . '/js/structfieldstorage.js',
        drupal_get_path('module', 'structfieldstorage') . '/js/autocomplete.js'
      ),
    ),
  );
  
  // @TODO Add linkage that come from structEntities here.

  // Declare the fieldset and field for RDF type.
  $form['type'] = array(
    '#type' => 'fieldset',
    '#title' => t('Type'),
    '#group' => 'rdf_mappings',
    '#attributes' => array('class' => array('rdf-type')),
  );
  $form['type']['rdf_rdftype'] = array(
    '#type' => 'textarea',
    '#title' => t('RDF Type'),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'structfieldstorage') . '/structfieldstorage.js'),
    ),
  );

  // Declare the fieldset and field for RDF type.
  $form['type'] = array(
    '#type' => 'fieldset',
    '#title' => t('Type'),
    '#group' => 'rdf_mappings',
    '#attributes' => array('class' => array('rdf-field')),
  );
  $form['type']['rdf_rdftype'] = array(
    '#type' => 'textarea',
    '#title' => t('RDF Type'),
    '#attributes' => array('class' => array('rdf-field')),
  );

  // Add the field for RDF type.
  structfieldstorage_rdftype_fieldset($form['type']['rdf_rdftype'], $mapping);

  // Add the options for all other fields.
  foreach ($fields as $field_name => $field) {

    // skip title & body core fields, there are handled automatically later in the process
    if($field_name == 'title' || $field_name == 'body')
    {
      continue;
    }
    
    // Make sure this field does use the "structfieldstorage" storage type
    $storageType = db_query('SELECT storage_type FROM {field_config} WHERE field_name = :field_name LIMIT 1', array(':field_name' => $field_name))->fetchField();

    if($storageType != 'structfieldstorage' && !variable_get('structfieldstorage_show_all_fields_rdf_mapping', 0))
    {
      continue;
    }
    
    $label = $field['label'];
    
    $form[$field_name] = array(
      '#type' => 'fieldset',
      '#group' => 'rdf_mappings',
      '#title' => t('@label', array('@label' => $label)),
      '#attributes' => array('class' => array('rdf-field')),
    );
    
    structfieldstorage_predicate_fieldset($form[$field_name], $mapping, $field_name, $label);
  }

  $form['field_names'] = array('#type' => 'value', '#value' => array_keys($fields));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save mappings'));

  return $form;
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function structfieldstorage_form_field_ui_field_edit_form_alter(&$form, &$form_state) {

  $field_name = $form['#field']['field_name'];
  $instance = $form['instance'];
  $label = isset($instance['label']) ? $instance['label']['#default_value'] : $instance['field_name'];
  $entity_type = $instance['entity_type']['#value'];
  $mapping = rdf_mapping_load($entity_type, $instance['bundle']['#value']);

  $form['rdf'] = array(
    '#type' => 'fieldset',
    '#title' => t('@label RDF Mapping', array('@label' => $label)),
  );

  // add the predicate, etc fields
  structfieldstorage_predicate_fieldset($form['rdf'], $mapping, $field_name, $label);

  $form['submit']['#weight'] = 1;

  // add submit and validate handlers
  $form['#validate'] = array_merge($form['#validate'], array('structfieldstorage_form_field_ui_field_edit_form_validate'));
  $form['#submit'] = array_merge($form['#submit'], array('structfieldstorage_form_field_ui_field_edit_form_submit'));
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function structfieldstorage_form_node_type_form_alter(&$form, &$form_state) {
  $mapping = array();
  // If we are editing an existing content type, the mapping will be available
  // via the bundles in entity info.
  if ($bundle = $form['type']['#default_value']) {
    $entity = entity_get_info('node');
    $mapping = $entity['bundles'][$bundle]['rdf_mapping'];
  }

  $form['rdf_settings'] = array(
    '#type' => 'fieldset',
    '#title' => 'RDF Settings',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'additional_settings'
  );
  $form['rdf_settings']['rdf_rdftype'] = array(
    '#type' => 'fieldset',
    '#title' => 'RDF Type',
  );
  $form['rdf_settings']['rdf_title'] = array(
    '#type' => 'fieldset',
    '#title' => 'RDF Title Predicate',
  );

  // Add the RDF type field for the bundle.
  structfieldstorage_rdftype_fieldset($form['rdf_settings']['rdf_rdftype'], $mapping);
  
  //Add the predicate, datatype, property fields for the bundle title.
  structfieldstorage_predicate_fieldset($form['rdf_settings']['rdf_title'], $mapping, 'title', 'title');

  // add submit and validate handlers
  $form['#validate'] = array_merge($form['#validate'], array('structfieldstorage_form_node_type_form_validate'));
  $form['#submit'] = array_merge($form['#submit'], array('structfieldstorage_form_node_type_form_submit'));
}

/*
function structfieldstorage_form_field_ui_field_edit_form_validate($form, &$form_state) {
  $field_name = $form['#field']['field_name'];

  //_structfieldstorage_validate_terms($form_state, $field_name);
  //_structfieldstorage_validate_datatype($form_state, $field_name);
}
*/
/*
function structfieldstorage_admin_rdf_overview_form_validate($form, &$form_state) {
  // Validate bundle's RDF type(s).
  _structfieldstorage_validate_terms($form_state);

  // Validate title predicate(s).
  if ((isset($form_state['input']['rdf_title_type']))) {
    $field_name = 'title';
    _structfieldstorage_validate_terms($form_state, $field_name);
    //_structfieldstorage_validate_datatype($form_state, $field_name);
  }

  // Validate predicates for all fields.
  foreach ($form_state['values']['field_names'] as $field_name) {
    _structfieldstorage_validate_terms($form_state, $field_name);
    //_structfieldstorage_validate_datatype($form_state, $field_name);
  }
}
*/
/*
function structfieldstorage_form_node_type_form_validate($form, &$form_state) {
  // Validate bundle's RDF type(s).
  _structfieldstorage_validate_terms($form_state);

  // Validate title predicate(s).
  if ((isset($form_state['input']['rdf_title_type']))) {
    $field_name = 'title';
    _structfieldstorage_validate_terms($form_state, $field_name);
    //_structfieldstorage_validate_datatype($form_state, $field_name);
  }
}
*/

/**
 * Saves RDF mapping for all fields and node type.
 */
function structfieldstorage_admin_rdf_overview_form_submit($form, &$form_state) {
  $entity_type = $form_state['build_info']['args'][0];
  // @todo somehow $form_state['build_info']['args'][1] is an object for the
  // node RDF mapping but a string 'user' for the user RDF mapping.
  if (isset($form_state['build_info']['args'][1]->type)) {
    $bundle = $form_state['build_info']['args'][1]->type;
  }
  elseif ($form_state['build_info']['args'][0] == 'taxonomy_term') {
    $bundle = $form_state['build_info']['args'][1]->machine_name;
  }
  else {
    $bundle = $form_state['build_info']['args'][1];
  }
  _structfieldstorage_mapping_save($form_state, $entity_type, $bundle, 'title');

  foreach ($form_state['values']['field_names'] as $field_name) {
    _structfieldstorage_mapping_save($form_state, $entity_type, $bundle, $field_name);
  }
  drupal_set_message(t('RDF mappings have been saved.'), 'status');
}

/**
 * Saves RDF mapping for individual field.
 */
function structfieldstorage_form_field_ui_field_edit_form_submit($form, &$form_state) {
  $entity_type = $form['instance']['entity_type']['#value'];
  $bundle = $form['instance']['bundle']['#value'];
  $field_name = $form['#field']['field_name'];

  _structfieldstorage_mapping_save($form_state, $entity_type, $bundle, $field_name);
}

/**
 * Saves RDF mapping for Title field.
 */
function structfieldstorage_form_node_type_form_submit($form, &$form_state) {
  $entity_type = 'node';
  $bundle = $form_state['input']['type'];
  // We only need to call _structfieldstorage_mapping_save once because it checks whether
  // rdf_type is set in the form before saving the field value.
  _structfieldstorage_mapping_save($form_state, $entity_type, $bundle, 'title');
}

/**
 * Menu callback for classes autocomplete
 */
 
function structfieldstorage_classes_autocomplete($query) {
  $networks = structconf_get_endpoints(NULL, FALSE, TRUE);

  $suggestions = array();
  
  $ontologies = variable_get('construct_structentities_ontologies_selected', array());
  
  $ontologiesFilters = array();
  
  foreach($ontologies as $ontology)
  {
    if($ontology != NULL)
    {
      array_push($ontologiesFilters, $ontology);
    }
  }

  foreach($networks as $network)
  {
    $search = new SearchQuery($network->uri);
    
    if($query == '')
    {
      $search->excludeAggregates()
             ->sort('prefLabel', 'asc')
             ->items(15)
             ->attributeValuesFilters(Namespaces::$iron.'prefLabel', '*');
    }
    else
    {
      $search->excludeAggregates()
             ->sort('prefLabel', 'asc')
             ->items(15)
             ->attributeValuesFilters(Namespaces::$iron.'prefLabel', '*'.$query.'**');
    } 
    
    $search->datasetsFilters($ontologiesFilters)
           ->typeFilter(Namespaces::$owl.'Class')
           ->sourceInterface(variable_get("structsearchapi_settings_interface_name", ''))
           ->send(new DrupalQuerierExtension());
    
    if(!$search->isSuccessful())
    {
      watchdog('structentities', 
               'Can\'t get the list of things to autocomplete: [@error] @errorMsg - @errorDescription.', 
               array("@error" => $search->getStatus(),                 
                     "@errorMsg" => $search->getStatusMessage(),
                     "@errorDescription" => $search->getStatusMessageDescription()));                       
                         
    }
    else
    {           
      $resultset = $search->getResultset();
     
      $namespaces = new Namespaces();
     
      if(isset($resultset))
      {
        foreach($resultset->getResultset() as $dataset => $results)
        {
          foreach($results as $uri => $result)
          {
            $suggestions[$result['prefLabel'] . '   ('.$namespaces->getPrefixedUri($uri).')'] = '<b>'.$result['prefLabel'] . '</b>   <em>('.$namespaces->getPrefixedUri($uri).')</em>';
          }
        }
      }
      
      // We we got more than 10 results, we stop querying for more networks
      if(count($suggestions) > 10)
      {
        break;
      } 
    }   
  }

  drupal_json_output($suggestions);
}  

/**
 * Menu callback for datatype autocomplete
 *//*
function structfieldstorage_datatype_autocomplete($string) {

  //@todo Make sure datatype is only active if property is the relationship.

  $datatypes = 'xsd:string, xsd:boolean, xsd:decimal, xsd:float, xsd:double, xsd:dateTime, xsd:time, xsd:date, xsd:gYearMonth, xsd:gYear, xsd:gMonthDay, xsd:gDay, xsd:gMonth, xsd:hexBinary, xsd:base64Binary, xsd:anyURI, xsd:normalizedString, xsd:token, xsd:language, xsd:NMTOKEN, xsd:Name, xsd:NCName, xsd:integer, xsd:nonPositiveInteger, xsd:negativeInteger, xsd:long, xsd:int, xsd:short, xsd:byte, xsd:nonNegativeInteger, xsd:unsignedLong, xsd:unsignedInt, xsd:unsignedShort, xsd:unsignedByte, xsd:positiveInteger';

  $datatypes = explode(', ', $datatypes);
  $matches = array();

  foreach($datatypes as $datatype) {
    if (preg_match("/^$string/", $datatype))
      $matches[$datatype] = $datatype;
  }

  drupal_json_output($matches);
} */

/**
 * Menu callback for predicates autocomplete
 */
function structfieldstorage_literal_predicates_autocomplete($query) {
  $networks = structconf_get_endpoints(NULL, FALSE, TRUE);

  $suggestions = array();
  
  $ontologies = variable_get('construct_structentities_ontologies_selected', array());
  
  $ontologiesFilters = array();
  
  foreach($ontologies as $ontology)
  {
    if($ontology != NULL)
    {
      array_push($ontologiesFilters, $ontology);
    }
  }

  foreach($networks as $network)
  {
    $search = new SearchQuery($network->uri);
    
    if($query == '')
    {
      $search->excludeAggregates()
             ->items(15)
             ->sort('prefLabel', 'asc')
             ->attributeValuesFilters(Namespaces::$iron.'prefLabel', '*');
    }
    else
    {
      $search->excludeAggregates()
             ->items(15)
             ->sort('prefLabel', 'asc')
             ->attributeValuesFilters(Namespaces::$iron.'prefLabel', '*'.$query.'**');
    } 
    
    $search->datasetsFilters($ontologiesFilters)
           ->typeFilter(Namespaces::$owl.'DatatypeProperty')
           ->typeFilter(Namespaces::$owl.'AnnotationProperty')
           ->sourceInterface(variable_get("structsearchapi_settings_interface_name", ''))
           ->send(new DrupalQuerierExtension());

    if(!$search->isSuccessful())
    {
      watchdog('structentities', 
               'Can\'t get the list of things to autocomplete: [@error] @errorMsg - @errorDescription.', 
               array("@error" => $search->getStatus(),                 
                     "@errorMsg" => $search->getStatusMessage(),
                     "@errorDescription" => $search->getStatusMessageDescription()));                       
                         
    }
    else
    {           
      $resultset = $search->getResultset();
     
      $namespaces = new Namespaces();
      
      if(isset($resultset))
      {
        foreach($resultset->getResultset() as $dataset => $results)
        {  
          foreach($results as $uri => $result)
          {
            $valueType = 'text';
            
            switch($result['type'][0])
            {
              case Namespaces::$owl.'DatatypeProperty':
                $valueType = 'text';
              break;
              case Namespaces::$owl.'AnnotationProperty':
                $valueType = 'text';
              break;
              case Namespaces::$owl.'ObjectProperty':
                $valueType = 'relation';
              break;
            }            
            
            $suggestions[$result['prefLabel'] /*. ' ['.$valueType.'] '*/ .' ('.$namespaces->getPrefixedUri($uri).')'] = '<b>'.$result['prefLabel'] . '</b> '. /*'<b>['.$valueType.']</b>'.*/ ' <em>('.$namespaces->getPrefixedUri($uri).')</em>';
          }
        }
      }
      
      // We we got more than 10 results, we stop querying for more networks
      if(count($suggestions) > 10)
      {
        break;
      } 
    }   
  }

  drupal_json_output($suggestions);
}

/**
 * Menu callback for predicates autocomplete
 */
function structfieldstorage_relation_predicates_autocomplete($query) {
  $networks = structconf_get_endpoints(NULL, FALSE, TRUE);

  $suggestions = array();
  
  $ontologies = variable_get('construct_structentities_ontologies_selected', array());
  
  $ontologiesFilters = array();
  
  foreach($ontologies as $ontology)
  {
    if($ontology != NULL)
    {
      array_push($ontologiesFilters, $ontology);
    }
  }

  foreach($networks as $network)
  {
    $search = new SearchQuery($network->uri);
    
    if($query == '')
    {
      $search->excludeAggregates()
             ->items(15)
             ->sort('prefLabel', 'asc')
             ->attributeValuesFilters(Namespaces::$iron.'prefLabel', '*');
    }
    else
    {
      $search->excludeAggregates()
             ->items(15)
             ->sort('prefLabel', 'asc')
             ->attributeValuesFilters(Namespaces::$iron.'prefLabel', '*'.$query.'**');
    } 
    
    $search->datasetsFilters($ontologiesFilters)
           ->typeFilter(Namespaces::$owl.'ObjectProperty')
           ->sourceInterface(variable_get("structsearchapi_settings_interface_name", ''))
           ->send(new DrupalQuerierExtension());

    if(!$search->isSuccessful())
    {
      watchdog('structentities', 
               'Can\'t get the list of things to autocomplete: [@error] @errorMsg - @errorDescription.', 
               array("@error" => $search->getStatus(),                 
                     "@errorMsg" => $search->getStatusMessage(),
                     "@errorDescription" => $search->getStatusMessageDescription()));                       
                         
    }
    else
    {           
      $resultset = $search->getResultset();
     
      $namespaces = new Namespaces();
      
      if(isset($resultset))
      {
        foreach($resultset->getResultset() as $dataset => $results)
        {  
          foreach($results as $uri => $result)
          {
            $valueType = 'text';
            
            switch($result['type'][0])
            {
              case Namespaces::$owl.'DatatypeProperty':
                $valueType = 'text';
              break;
              case Namespaces::$owl.'AnnotationProperty':
                $valueType = 'text';
              break;
              case Namespaces::$owl.'ObjectProperty':
                $valueType = 'relation';
              break;
            }            
            
            $suggestions[$result['prefLabel'] /*. ' ['.$valueType.'] '*/ .' ('.$namespaces->getPrefixedUri($uri).')'] = '<b>'.$result['prefLabel'] . '</b> '. /*'<b>['.$valueType.']</b>'.*/ ' <em>('.$namespaces->getPrefixedUri($uri).')</em>';
          }
        }
      }
      
      // We we got more than 10 results, we stop querying for more networks
      if(count($suggestions) > 10)
      {
        break;
      } 
    }   
  }

  drupal_json_output($suggestions);
}


/**
 * appends the rdf form fields to a fieldset dedicated to a fields.module field
 */
function structfieldstorage_predicate_fieldset(&$fieldset, $mapping, $field_name, $label) {
  $fieldInfo = field_info_field($field_name);
  $fieldInstance = field_read_instance('node', $field_name, $fieldInfo['bundles']['node'][0]);
    
  // Check if the field type and/or the field widget that is used is the kind of field
  // that reference another entity. If it is the case, then we make sure that the
  // RDF Mapping user interface only shows owl:ObjectProperty to the user. Otherwise
  // we display owl:DatatypeProperty and owl:AnnotationProperty to the users.
  if($fieldInfo['type'] == 'entityreference' ||
     ($fieldInfo['type'] == 'text' && $fieldInstance['widget']['type'] == 'struct_lookup_field') ||
     ($fieldInfo['type'] == 'text' && $fieldInstance['widget']['type'] == 'struct_lookup_field_with_scones_suggest') ||
     ($fieldInfo['type'] == 'text' && $fieldInstance['widget']['type'] == 'struct_lookup_field_with_scones_tagging'))
  {
    $fieldset['rdf_'. $field_name .'_predicates'] = array(
      '#type' => 'textfield',
      '#autocomplete_path' => 'structfieldstorage/relation_predicates/autocomplete',
      '#title' => t('RDF Predicates'),
      '#default_value' => (empty($mapping[$field_name]['structfieldstorage_ui']) ? '' : $mapping[$field_name]['structfieldstorage_ui']),
      '#description' => t('Select the properties to use for mapping this field to in RDF. The list of properties can be managed by the <a target="_blank" href="/construct/ontology">structOntology module</a>. Create the new Datatype properties that is required for the mapping. If you don\'t have access to this tool, ask your system administrator to add it for you.',
        array(
          '%label' => $label,
          '%predicates' => 'foaf:familyName, foaf:lastName'
        )
      ),
      '#resizable' => FALSE,
    );
  }
  else            
  {
    $fieldset['rdf_'. $field_name .'_predicates'] = array(
      '#type' => 'textfield',
      '#autocomplete_path' => 'structfieldstorage/literal_predicates/autocomplete',
      '#title' => t('RDF Predicates'),
      '#default_value' => (empty($mapping[$field_name]['structfieldstorage_ui']) ? '' : $mapping[$field_name]['structfieldstorage_ui']),
      '#description' => t('Select the properties to use for mapping this field to in RDF. The list of properties can be managed by the <a target="_blank" href="/construct/ontology">structOntology module</a>. Create the new Datatype properties that is required for the mapping. If you don\'t have access to this tool, ask your system administrator to add it for you.',
        array(
          '%label' => $label,
          '%predicates' => 'foaf:familyName, foaf:lastName'
        )
      ),
      '#resizable' => FALSE,
    );
  }
}

/**
 * Appends the RDF type fieldset. Because this fieldset is always on the same
 * page as fields created by structfieldstorage_predicate_fieldset(), which add the css and
 * js files, we do not add the files again.
 *
 */
function structfieldstorage_rdftype_fieldset(&$fieldset, $mapping) {
  $fieldset = array(
    '#type' => 'textfield',
    '#autocomplete_path' => 'structfieldstorage/classes/autocomplete',
    '#title' => t('RDF Type'),
    '#default_value' => (empty($mapping['rdftype']['structfieldstorage_ui']) ? '' : $mapping['rdftype']['structfieldstorage_ui']),
    '#description' => t('Select the type to use for mapping this bundle to in RDF. The list of types can be managed by the <a href="/construct/entities/ontologies/sync/classes">structEntities synchronization process</a>. The possible types exposed are constrained to the <a href="/construct/entities/ontologies/select">list of selected ontology configured here</a>.'),
  );
}

/**
 * Sets a form error if predicates don't validate.
 */
/*
function _structfieldstorage_validate_terms($form_state, $field_name = '') {
}
*/

/**
 * Sets a form error if datatype don't validate.
 */
/* 
function _structfieldstorage_validate_datatype($form_state, $field_name) {
}
*/

function structfieldstorage_extract_uri($value, $prefixed = TRUE)
{
  $namespaces = new Namespaces();

  $uri = substr($value, strpos($value, '(') + 1, strpos($value, ')') - strpos($value, '(') - 1);
    
  if($prefixed === TRUE)  
  {
    return $uri;
  }
  else
  {
    return($namespaces->getUnprefixedUri($uri));
  }
}

function structfieldstorage_extract_property_mapping_type($value)
{
  $namespaces = new Namespaces();

  $mapping_type = substr($value, strpos($value, '[') + 1, strpos($value, ']') - strpos($value, '[') - 1);
  
  return($mapping_type);
}

/**
 * Saves the mapping
 */
function _structfieldstorage_mapping_save($form_state, $entity_type, $bundle, $field_name) {
  $prevMapping = rdf_mapping_load($entity_type, $bundle);
  $mapping = rdf_mapping_load($entity_type, $bundle);
  
  // Set the RDF type if it is in this form.
  if (!empty($form_state['values']['rdf_rdftype'])) {
    
    $typeClass = structfieldstorage_extract_uri($form_state['values']['rdf_rdftype']);
    
    // Check if the type mapping changed and record that change in the structfieldstorage_pending_opts_fields table
    if(isset($prevMapping['rdftype']) && $prevMapping['rdftype'][0] != $typeClass)
    {
      db_insert('structfieldstorage_pending_opts_bundles')
                ->fields(array(
                  'bundle' => $bundle,
                  'rdfmapping' => $typeClass,
                  'prev_rdfmapping' => $prevMapping['rdftype'][0],
                  'operation' => 'changed',
                  'date' => REQUEST_TIME,
                  'executed' => 0
                ))
                ->execute();            
    }    
    
    $mapping['rdftype'] = array($typeClass);
    $mapping['rdftype']['structfieldstorage_ui'] = $form_state['values']['rdf_rdftype'];     
  }  
  
  foreach(array('type', 'datatype', 'predicates') as $key) 
  {
    // If there is form input for this key, set it in the mapping.
    if(!empty($form_state['values']['rdf_'. $field_name .'_' . $key])) 
    {
      $mappedPropertyUIString = $form_state['values']['rdf_'. $field_name .'_' . $key];
      $mappedProperty = structfieldstorage_extract_uri($mappedPropertyUIString);
      
      $mapping[$field_name][$key] = array($mappedProperty);
      
      if(structfieldstorage_extract_property_mapping_type($mappedPropertyUIString) == 'relation')
      {
        $mapping[$field_name]['type'] = 'rel';
      } 
      
      $mapping[$field_name]['structfieldstorage_ui'] = $mappedPropertyUIString;     
      
      // Check if the property mapping changed and record that change in the structfieldstorage_pending_opts_bundles table
      if(isset($prevMapping['rdftype']) && 
         isset($prevMapping[$field_name]) && 
         $prevMapping[$field_name]['predicates'][0] != $mappedProperty)
      {
        db_insert('structfieldstorage_pending_opts_fields')
                  ->fields(array(
                    'bundle' => $bundle,
                    'field' => $field_name,
                    'rdfmapping' => $mappedProperty,
                    'prev_rdfmapping' => $prevMapping[$field_name]['predicates'][0],
                    'operation' => 'changed',
                    'date' => REQUEST_TIME,
                    'executed' => 0
                  ))
                  ->execute();            
      }
    }
    // If there is no form input for this key, we need to remove it from the
    // mapping.
    else 
    {
      // If there are no predicates, there should be no mapping at all for the
      // field.
      if ($key == 'predicates') {
        unset($mapping[$field_name]);
      }
      else {
        unset($mapping[$field_name][$key]);
      }
    }
  }  

  $mapping_info = array(
    'mapping' => $mapping,
    'type' => $entity_type,
    'bundle' => $bundle
  );

  rdf_mapping_save($mapping_info);  
}   
  
?>
